@@ -13,13 +13,12 @@ import re
 import os
 import os.path
 import subprocess
-from colorama import init
-from colorama import Fore
-
-init()
 
 from deployment.models import YhatModel
 from deployment.save_session import save_function, _get_source
+from requirements import merge
+
+from .utils import progressbarify
 
 BASE_URI = "http://api.yhathq.com/"
 
@@ -67,7 +66,7 @@ class API(object):
         except Exception, e:
             raise e
 
-    def post(self, endpoint, params, data):
+    def post(self, endpoint, params, data, pb=False):
         """
         Parameters
         ----------
@@ -77,6 +76,8 @@ class API(object):
             querystring parameters for API call
         data: dictionary
             data you want transfered as JSON
+        pb: bool
+            do you want a progress bar?
 
         Returns
         -------
@@ -100,6 +101,8 @@ as a pandas DataFrame. If you're still having trouble, please contact:
 {URL}.""".format(URL="support@yhathq.com")
                 print msg
                 return
+            if pb:
+                data = progressbarify(data)
             response = urllib2.urlopen(req, data)
             rsp = response.read()
             try:
@@ -133,6 +136,7 @@ as a pandas DataFrame. If you're still having trouble, please contact:
         username = self.username if model_owner is None else model_owner
         ws_uri = ws_uri.format(BASE_URI=ws_base, USERNAME=username,
                                MODEL_NAME=model_name)
+        ws_uri = "%s?username=%s&apikey=%s" % (ws_uri, self.username, self.apikey)
         ws = websocket.create_connection(ws_uri)
         auth = {
             "username": self.username,
@@ -237,7 +241,7 @@ class Yhat(API):
                 data = dict(zip(data.columns, data_values))
         return data
 
-    def predict(self, model, data, model_owner=None):
+    def predict(self, model, data, model_owner=None, raw_input=False):
         """
         Executes a single prediction via the Yhat API.
 
@@ -250,6 +254,9 @@ class Yhat(API):
             a dataframe
         model_owner: string
             username of the model owner for shared models. optional
+        raw_input: bool
+            whether or not to attempt to parse the incoming JSON. for calling R 
+            models only
 
         Returns
         -------
@@ -259,6 +266,8 @@ class Yhat(API):
         data = self._convert_to_json(data)
         q = self.q
         q['model'] = model
+        if raw_input==True:
+            q['raw_input'] = True
         model_user = self.username if model_owner is None else model_owner
         if self.base_uri != BASE_URI:
             endpoint = "%s/models/%s/" % (model_user, model)
@@ -348,19 +357,13 @@ need to connect to the server first. try running "connect_to_socket"
         bundle["language"] = "python"
         bundle["modelname"] = name
         bundle["className"] = model.__name__
-        reqs = getattr(model, "REQUIREMENTS", "")
-        if isinstance(reqs, list):
-            reqs = '\n'.join(reqs)
-        bundle["reqs"] = reqs
-        # make sure we freeze Yhat so we're sure we're using the right version
-        # this makes it a lot easier to upgrade the client
-        import yhat
-        bundle["reqs"] += '\n' + "yhat==" + yhat.__version__
-        bundle["reqs"] = bundle["reqs"].strip().replace(
-            '"', '').replace("'", "")
+        bundle["reqs"] = "\n".join(
+            str(r) for r in merge(session, getattr(model, "REQUIREMENTS", ""))
+        )
+
         return bundle
 
-    def deploy(self, name, model, session, sure=False, packages=[]):
+    def deploy(self, name, model, session, sure=False, packages=[], patch=None):
         """
         Deploys your model to a Yhat server
 
@@ -391,6 +394,9 @@ need to connect to the server first. try running "connect_to_socket"
                 sys.exit()
         bundle = self._extract_model(name, model, session)
         bundle['packages'] = packages
+        if isinstance(patch, str)==True:
+            patch = "\n".join([line.strip() for line in patch.strip().split('\n')])
+            bundle['code'] = patch + "\n" + bundle['code']
         if self._check_obj_size(bundle) is False:
             # we're not going to deploy; model is too big, but let's give the
             # user the option to upload it manually
@@ -401,10 +407,12 @@ need to connect to the server first. try running "connect_to_socket"
                 self.deploy_to_file(name, model, session)
         else:
             # upload the model to the server
-            # TODO: upload w/ a progress bar
-            return self.post("deployer/model", self.q, bundle)
+            print "Uploading model data"
+            data = self.post("deployer/model", self.q, bundle, pb=True)
+            print "Model uploaded"
+            return data
 
-    def deploy_to_file(self, name, model, session, compress=True, packages=[]):
+    def deploy_to_file(self, name, model, session, compress=True, packages=[], patch=None):
         """
         Bundles a local version of your model that can be manually uploaded to
         the server.
@@ -426,6 +434,9 @@ need to connect to the server first. try running "connect_to_socket"
         bundle = self._extract_model(name, model, session)
         bundle['apikey'] = self.apikey
         bundle['packages'] = packages
+        if isinstance(patch, str)==True:
+            patch = "\n".join([line.strip() for line in patch.strip().split('\n')])
+            bundle['code'] = patch + "\n" + bundle['code']
         filename = "%s.yhat" % name
         with open(filename, "w") as f:
             bundle = json.dumps(bundle)
@@ -438,8 +449,7 @@ need to connect to the server first. try running "connect_to_socket"
         msg = "To deploy, visit %s and upload %s."
         upload_url = os.path.join(self.base_uri, "model", "upload")
         msg = msg % (upload_url, "%s.yhat" % name)
-        print Fore.CYAN + msg
-        print Fore.RESET
+        print msg
         return filename
 
     def deploy_with_scp(self, name, model, sessions,
@@ -465,24 +475,3 @@ need to connect to the server first. try running "connect_to_socket"
         /var/yhat/headquarters/uploads/'""" % (pem_path, server_uri, filename)
         subprocess.check_call(ssh_cmd, shell=True)
         os.remove(filename)
-
-if __name__ == "__main__":
-    import pandas as pd
-    df = pd.DataFrame({
-        "x": range(10),
-        "y": range(10)
-    })
-    yh = Yhat("blah", "blue")
-    print yh._convert_to_json({"x": 1})
-    print yh._convert_to_json(df)
-
-    from yhat import credentials
-    creds = credentials.read()
-
-    yh = Yhat(creds['username'], creds['apikey'], "http://localhost:8080/")
-
-    print yh.predict("mymodel", {"beer": "Coors Light"})
-    yh.connect_to_socket("mymodel")
-    print yh.predict_ws({"beer": "Coors Light"})
-    for item in yh.yield_results():
-        print item
